package org.jarbframework.populator.excel.mapping.importer;

import org.jarbframework.populator.excel.mapping.ValueConversionService;
import org.jarbframework.populator.excel.metamodel.EntityDefinition;
import org.jarbframework.populator.excel.metamodel.PropertyDefinition;
import org.jarbframework.populator.excel.workbook.Workbook;

/**
 * Function which is responsible for saving values to ExcelRecords, put here because the ExcelRow class was getting quite large.
 * @author Willem Eppen
 * @author Sander Benschop
 *
 */
public final class StoreExcelRecordValue {
    private StoreColumn storeColumn;

    public StoreExcelRecordValue(ValueConversionService conversionService) {
        this.storeColumn = new StoreColumn(conversionService);
    }

    /**
     * Calls functions to save data. There are three possible types: Column, JoinColumn and JoinTable. 
     * Each have their own saving rules, for each one the appropriate function is called.
     * @param excel Representation of excel file
     * @param entityDefinition Definition representing an entity which is used to determine columnPosition
     * @param columnDefinition ColumnDefinition is the superclass of Column, JoinColumn and JoinTable.
     * @param rowPosition Vertical position number of the excelRecord
     * @param excelRow ExcelRow to save to.
     * @throws NoSuchFieldException Thrown when a field is not available
     */
    public void storeValue(Workbook excel, EntityDefinition<?> entityDefinition, PropertyDefinition columnDefinition, Integer rowPosition, ExcelRow excelRow) {
        switch (columnDefinition.getDatabaseType()) {
        case COLUMN:
            storeColumn.storeValue(excel, entityDefinition, columnDefinition, rowPosition, excelRow);
            break;
        case COLLECTION_REFERENCE:
            StoreJoinTable.storeValue(excel, entityDefinition, columnDefinition, rowPosition, excelRow);
            break;
        case REFERENCE:
            StoreJoinColumn.storeValue(excel, entityDefinition, columnDefinition, rowPosition, excelRow);
            break;
        case INVERSED_REFERENCE:
            StoreInversedJoinColumn.storeValue(excel, entityDefinition, columnDefinition, rowPosition, excelRow);
            break;
        default:
            throw new RuntimeException("Cannot store ExcelRow value of ColumnDefinition from field '" + columnDefinition.getName()
                    + "'. It is of an unknown DatabaseType.");
        }
    }

}
